@@ -27,6 +27,7 @@ from .buckets import (
     BlobCommonPrefix,
     BlobObject,
     Bucket,
+    BucketCredentials,
     BucketEntry,
     Buckets,
     PersistentBucketCredentials,
@@ -84,7 +85,7 @@ from .tracing import gen_trace_id
 from .users import Action, Permission, Share, Users
 from .utils import _ContextManager, find_project_root
 
-__version__ = "21.9.2"
+__version__ = "21.10.0"
 
 
 __all__ = (
@@ -100,6 +101,7 @@ __all__ = (
     "BlobCommonPrefix",
     "BlobObject",
     "Bucket",
+    "BucketCredentials",
     "BucketEntry",
     "Buckets",
     "CONFIG_ENV_NAME",
