@@ -1,9 +1,10 @@
 from dataclasses import dataclass
+from datetime import datetime
 from decimal import Decimal
 from enum import Enum, unique
 from typing import Any, Dict, List, Mapping, Optional
 
-from neuro_sdk.users import Quota
+from dateutil.parser import isoparse
 
 from .config import Config
 from .core import _Core
@@ -21,11 +22,42 @@ class _ClusterUserRoleType(str, Enum):
         return self.value
 
 
+@dataclass(frozen=True)
+class _UserInfo:
+    email: str
+    first_name: Optional[str]
+    last_name: Optional[str]
+    created_at: Optional[datetime]
+
+    @property
+    def full_name(self) -> str:
+        if self.first_name and self.last_name:
+            return f"{self.first_name} {self.last_name}"
+        if self.first_name:
+            return self.first_name
+        if self.last_name:
+            return self.last_name
+        return ""
+
+
+@dataclass(frozen=True)
+class _Quota:
+    total_running_jobs: Optional[int] = None
+
+
+@dataclass(frozen=True)
+class _Balance:
+    credits: Optional[Decimal] = None
+    spent_credits: Decimal = Decimal("0")
+
+
 @dataclass(frozen=True)
 class _ClusterUser:
     user_name: str
     role: _ClusterUserRoleType
-    quota: Quota
+    quota: _Quota
+    user_info: _UserInfo
+    balance: _Balance
 
 
 @dataclass(frozen=True)
@@ -117,7 +149,9 @@ class _Admin(metaclass=NoPublicConstructor):
         cluster_name = cluster_name or self._config.cluster_name
         url = self._config.admin_url / "clusters" / cluster_name / "users"
         auth = await self._config._api_auth()
-        async with self._core.request("GET", url, auth=auth) as resp:
+        async with self._core.request(
+            "GET", url, auth=auth, params={"with_user_info": "true"}
+        ) as resp:
             res = await resp.json()
             return [_cluster_user_from_api(payload) for payload in res]
 
@@ -130,7 +164,9 @@ class _Admin(metaclass=NoPublicConstructor):
         user_name = user_name or self._config.username
         url = self._config.admin_url / "clusters" / cluster_name / "users" / user_name
         auth = await self._config._api_auth()
-        async with self._core.request("GET", url, auth=auth) as resp:
+        async with self._core.request(
+            "GET", url, auth=auth, params={"with_user_info": "true"}
+        ) as resp:
             res = await resp.json()
             return _cluster_user_from_api(res)
 
@@ -141,7 +177,9 @@ class _Admin(metaclass=NoPublicConstructor):
         payload = {"user_name": user_name, "role": role}
         auth = await self._config._api_auth()
 
-        async with self._core.request("POST", url, json=payload, auth=auth) as resp:
+        async with self._core.request(
+            "POST", url, json=payload, auth=auth, params={"with_user_info": "true"}
+        ) as resp:
             payload = await resp.json()
             return _cluster_user_from_api(payload)
 
@@ -157,7 +195,6 @@ class _Admin(metaclass=NoPublicConstructor):
         self,
         cluster_name: str,
         user_name: str,
-        credits: Optional[Decimal],
         total_running_jobs: Optional[int],
     ) -> _ClusterUser:
         url = (
@@ -170,7 +207,6 @@ class _Admin(metaclass=NoPublicConstructor):
         )
         payload = {
             "quota": {
-                "credits": str(credits) if credits is not None else None,
                 "total_running_jobs": total_running_jobs,
             },
         }
@@ -178,15 +214,17 @@ class _Admin(metaclass=NoPublicConstructor):
 
         auth = await self._config._api_auth()
 
-        async with self._core.request("PATCH", url, json=payload, auth=auth) as resp:
+        async with self._core.request(
+            "PATCH", url, json=payload, auth=auth, params={"with_user_info": "true"}
+        ) as resp:
             payload = await resp.json()
             return _cluster_user_from_api(payload)
 
-    async def add_user_quota(
+    async def set_user_credits(
         self,
         cluster_name: str,
         user_name: str,
-        additional_credits: Optional[Decimal],
+        credits: Optional[Decimal],
     ) -> _ClusterUser:
         url = (
             self._config.admin_url
@@ -194,19 +232,40 @@ class _Admin(metaclass=NoPublicConstructor):
             / cluster_name
             / "users"
             / user_name
-            / "quota"
+            / "balance"
         )
         payload = {
-            "additional_quota": {
-                "credits": str(additional_credits) if additional_credits else None,
-            },
-        }
-        payload["additional_quota"] = {
-            k: v for k, v in payload["additional_quota"].items() if v is not None
+            "credits": str(credits) if credits is not None else None,
         }
+
         auth = await self._config._api_auth()
 
-        async with self._core.request("PATCH", url, json=payload, auth=auth) as resp:
+        async with self._core.request(
+            "PATCH", url, json=payload, auth=auth, params={"with_user_info": "true"}
+        ) as resp:
+            payload = await resp.json()
+            return _cluster_user_from_api(payload)
+
+    async def add_user_credits(
+        self,
+        cluster_name: str,
+        user_name: str,
+        additional_credits: Decimal,
+    ) -> _ClusterUser:
+        url = (
+            self._config.admin_url
+            / "clusters"
+            / cluster_name
+            / "users"
+            / user_name
+            / "balance"
+        )
+        payload = {"additional_credits": str(additional_credits)}
+        auth = await self._config._api_auth()
+
+        async with self._core.request(
+            "PATCH", url, json=payload, auth=auth, params={"with_user_info": "true"}
+        ) as resp:
             payload = await resp.json()
             return _cluster_user_from_api(payload)
 
@@ -219,15 +278,34 @@ class _Admin(metaclass=NoPublicConstructor):
             return await resp.json()
 
 
+def _user_info_from_api(payload: Dict[str, Any]) -> _UserInfo:
+    created_at = payload.get("created_at")
+    if created_at is not None:
+        created_at = isoparse(created_at)
+    return _UserInfo(
+        email=payload["email"],
+        first_name=payload.get("first_name"),
+        last_name=payload.get("last_name"),
+        created_at=created_at,
+    )
+
+
 def _cluster_user_from_api(payload: Dict[str, Any]) -> _ClusterUser:
     quota_dict = payload.get("quota", {})
+    balance_dict = payload.get("balance", {})
     return _ClusterUser(
         user_name=payload["user_name"],
         role=_ClusterUserRoleType(payload["role"]),
-        quota=Quota(
-            Decimal(quota_dict["credits"]) if "credits" in quota_dict else None,
-            quota_dict.get("total_running_jobs"),
+        quota=_Quota(
+            total_running_jobs=quota_dict.get("total_running_jobs"),
         ),
+        balance=_Balance(
+            credits=Decimal(balance_dict["credits"])
+            if "credits" in balance_dict
+            else None,
+            spent_credits=Decimal(balance_dict["spent_credits"]),
+        ),
+        user_info=_user_info_from_api(payload["user_info"]),
     )
 
 
