@@ -188,12 +188,13 @@ class Config(metaclass=NoPublicConstructor):
         return self._config_data.admin_url
 
     @property
-    def monitoring_url(self) -> URL:
-        return self._cluster.monitoring_url
+    def service_accounts_url(self) -> URL:
+        # TODO: use URL returned from server when available
+        return self._config_data.url / "service_accounts"
 
     @property
-    def blob_storage_url(self) -> URL:
-        return self._cluster.blob_storage_url
+    def monitoring_url(self) -> URL:
+        return self._cluster.monitoring_url
 
     @property
     def storage_url(self) -> URL:
@@ -211,6 +212,10 @@ class Config(metaclass=NoPublicConstructor):
     def disk_api_url(self) -> URL:
         return self._cluster.disks_url
 
+    @property
+    def bucket_api_url(self) -> URL:
+        return self._cluster.buckets_url
+
     async def token(self) -> str:
         token = self._config_data.auth_token
         if not token.is_expired():
@@ -277,7 +282,8 @@ def load_user_config(path: Path) -> Mapping[str, Any]:
 def _open_db_rw(
     path: Path, suppress_errors: bool = True
 ) -> Iterator[sqlite3.Connection]:
-    path.mkdir(0o700, parents=True, exist_ok=True)
+    path.mkdir(0o700, parents=True, exist_ok=True)  # atomically set proper bits
+    path.chmod(0o700)  # fix security if config folder already exists
 
     config_file = path / "db"
     with sqlite3.connect(str(config_file)) as db:
@@ -419,11 +425,11 @@ def _deserialize_clusters(payload: Dict[str, Any]) -> Dict[str, Cluster]:
             name=cluster_config["name"],
             registry_url=URL(cluster_config["registry_url"]),
             storage_url=URL(cluster_config["storage_url"]),
-            blob_storage_url=URL(cluster_config["blob_storage_url"]),
             users_url=URL(cluster_config["users_url"]),
             monitoring_url=URL(cluster_config["monitoring_url"]),
             secrets_url=URL(cluster_config["secrets_url"]),
             disks_url=URL(cluster_config["disks_url"]),
+            buckets_url=URL(cluster_config["buckets_url"]),
             presets=dict(
                 _deserialize_resource_preset(data)
                 for data in cluster_config.get("presets", [])
@@ -534,11 +540,11 @@ def _serialize_clusters(clusters: Mapping[str, Cluster]) -> str:
             "name": cluster.name,
             "registry_url": str(cluster.registry_url),
             "storage_url": str(cluster.storage_url),
-            "blob_storage_url": str(cluster.blob_storage_url),
             "users_url": str(cluster.users_url),
             "monitoring_url": str(cluster.monitoring_url),
             "secrets_url": str(cluster.secrets_url),
             "disks_url": str(cluster.disks_url),
+            "buckets_url": str(cluster.buckets_url),
             "presets": [
                 _serialize_resource_preset(name, preset)
                 for name, preset in cluster.presets.items()
