@@ -17,11 +17,11 @@ from typing import (
     Optional,
     Sequence,
     Set,
-    Union,
     overload,
 )
 
 import aiohttp
+import aiohttp.hdrs
 import attr
 from aiodocker.exceptions import DockerError
 from aiohttp import WSMsgType, WSServerHandshakeError
@@ -38,6 +38,7 @@ from .abc import (
 )
 from .config import Config
 from .core import _Core
+from .errors import NDJSONError, ResourceNotFound
 from .images import (
     _DummyProgress,
     _raise_on_error_chunk,
@@ -50,7 +51,7 @@ from .url_utils import (
     normalize_secret_uri,
     normalize_storage_path_uri,
 )
-from .utils import NoPublicConstructor
+from .utils import NoPublicConstructor, asyncgeneratorcontextmanager
 
 if sys.version_info >= (3, 7):  # pragma: no cover
     from contextlib import asynccontextmanager
@@ -97,15 +98,17 @@ class JobStatus(str, enum.Enum):
 
     @property
     def is_pending(self) -> bool:
-        return self in (self.PENDING, self.SUSPENDED)
+        cls = type(self)
+        return self in (cls.PENDING, cls.SUSPENDED)
 
     @property
     def is_running(self) -> bool:
-        return self == self.RUNNING
+        return self == type(self).RUNNING
 
     @property
     def is_finished(self) -> bool:
-        return self in (self.SUCCEEDED, self.FAILED, self.CANCELLED)
+        cls = type(self)
+        return self in (cls.SUCCEEDED, cls.FAILED, cls.CANCELLED)
 
     @classmethod
     def items(cls) -> Set["JobStatus"]:
@@ -119,6 +122,9 @@ class JobStatus(str, enum.Enum):
     def finished_items(cls) -> Set["JobStatus"]:
         return {item for item in cls.items() if item.is_finished}
 
+    __format__ = str.__format__
+    __str__ = str.__str__
+
 
 @dataclass(frozen=True)
 class HTTPPort:
@@ -191,6 +197,13 @@ class JobRestartPolicy(str, enum.Enum):
         return repr(self.value)
 
 
+@dataclass(frozen=True)
+class JobDescriptionInternal:
+    materialized: bool = False
+    being_dropped: bool = False
+    logs_removed: bool = False
+
+
 @dataclass(frozen=True)
 class JobDescription:
     id: str
@@ -214,6 +227,7 @@ class JobDescription:
     preset_name: Optional[str] = None
     preemptible_node: bool = False
     privileged: bool = False
+    _internal: JobDescriptionInternal = JobDescriptionInternal()
 
 
 @dataclass(frozen=True)
@@ -225,23 +239,19 @@ class JobTelemetry:
     gpu_memory: Optional[float] = None
 
 
-@dataclass(frozen=True)
-class ExecInspect:
-    id: str
-    running: bool
-    exit_code: int
-    job_id: str
-    tty: bool
-    entrypoint: str
-    command: str
-
-
 @dataclass(frozen=True)
 class Message:
     fileno: int
     data: bytes
 
 
+class StdStreamError(Exception):
+    def __init__(self, exit_code: int) -> None:
+        super().__init__(f"Stream finished with exit code {exit_code}")
+
+        self.exit_code = exit_code
+
+
 class StdStream:
     def __init__(self, ws: aiohttp.ClientWebSocketResponse) -> None:
         self._ws = ws
@@ -258,12 +268,25 @@ class StdStream:
         if msg.type in (WSMsgType.CLOSE, WSMsgType.CLOSING, WSMsgType.CLOSED):
             self._closing = True
             return None
+        if msg.data[0] == 3:
+            try:
+                details = json.loads(msg.data[1:])
+                exit_code = details["exit_code"]
+            except Exception:
+                exit_code = 1
+            self._closing = True
+            raise StdStreamError(exit_code)
         return Message(msg.data[0], msg.data[1:])
 
     async def write_in(self, data: bytes) -> None:
         if self._closing:
             return
-        await self._ws.send_bytes(data)
+        await self._ws.send_bytes(b"\x00" + data)
+
+    async def resize(self, h: int, w: int) -> None:
+        if self._closing:
+            return
+        await self._ws.send_bytes(b"\x04" + f'{{"height":{h},"width":{w}}}'.encode())
 
 
 class Jobs(metaclass=NoPublicConstructor):
@@ -272,6 +295,11 @@ class Jobs(metaclass=NoPublicConstructor):
         self._config = config
         self._parse = parse
 
+    def _get_monitoring_url(self, cluster_name: Optional[str]) -> URL:
+        if cluster_name is None:
+            cluster_name = self._config.cluster_name
+        return self._config.get_cluster(cluster_name).monitoring_url
+
     async def run(
         self,
         container: Container,
@@ -288,7 +316,7 @@ class Jobs(metaclass=NoPublicConstructor):
     ) -> JobDescription:
         url = self._config.api_url / "jobs"
         payload = _job_to_api(
-            config=self._config,
+            cluster_name=self._config.cluster_name,
             name=name,
             tags=tags,
             description=description,
@@ -324,6 +352,7 @@ class Jobs(metaclass=NoPublicConstructor):
         *,
         image: RemoteImage,
         preset_name: str,
+        cluster_name: Optional[str] = None,
         entrypoint: Optional[str] = None,
         command: Optional[str] = None,
         working_dir: Optional[str] = None,
@@ -362,7 +391,7 @@ class Jobs(metaclass=NoPublicConstructor):
             shm=shm,
         )
         payload = _job_to_api(
-            config=self._config,
+            cluster_name=cluster_name or self._config.cluster_name,
             name=name,
             preset_name=preset_name,
             tags=tags,
@@ -380,6 +409,7 @@ class Jobs(metaclass=NoPublicConstructor):
             res = await resp.json()
             return _job_description_from_api(res, self._parse)
 
+    @asyncgeneratorcontextmanager
     async def list(
         self,
         *,
@@ -391,6 +421,10 @@ class Jobs(metaclass=NoPublicConstructor):
         until: Optional[datetime] = None,
         reverse: bool = False,
         limit: Optional[int] = None,
+        cluster_name: Optional[str] = None,
+        _materialized: Optional[bool] = None,
+        _being_dropped: Optional[bool] = False,
+        _logs_removed: Optional[bool] = False,
     ) -> AsyncIterator[JobDescription]:
         url = self._config.api_url / "jobs"
         headers = {"Accept": "application/x-ndjson"}
@@ -412,11 +446,19 @@ class Jobs(metaclass=NoPublicConstructor):
             if until.tzinfo is None:
                 until = until.astimezone(timezone.utc)
             params.add("until", until.isoformat())
-        params["cluster_name"] = self._config.cluster_name
+        if cluster_name is None:
+            cluster_name = self._config.cluster_name
+        params["cluster_name"] = cluster_name
         if reverse:
             params.add("reverse", "1")
         if limit is not None:
             params.add("limit", str(limit))
+        if _materialized is not None:
+            params.add("materialized", str(_materialized))
+        if _being_dropped is not None:
+            params.add("being_dropped", str(_being_dropped))
+        if _logs_removed is not None:
+            params.add("logs_removed", str(_logs_removed))
         auth = await self._config._api_auth()
         async with self._core.request(
             "GET", url, headers=headers, params=params, auth=auth
@@ -425,7 +467,7 @@ class Jobs(metaclass=NoPublicConstructor):
                 async for line in resp.content:
                     server_message = json.loads(line)
                     if "error" in server_message:
-                        raise Exception(server_message["error"])
+                        raise NDJSONError(server_message["error"])
                     yield _job_description_from_api(server_message, self._parse)
             else:
                 ret = await resp.json()
@@ -449,8 +491,29 @@ class Jobs(metaclass=NoPublicConstructor):
             # an error is raised for status >= 400
             return None  # 201 status code
 
-    async def monitor(self, id: str) -> AsyncIterator[bytes]:
-        url = self._config.monitoring_url / id / "log"
+    @asyncgeneratorcontextmanager
+    async def monitor(
+        self,
+        id: str,
+        *,
+        cluster_name: Optional[str] = None,
+        since: Optional[datetime] = None,
+        timestamps: bool = False,
+        separator: Optional[str] = None,
+        debug: bool = False,
+    ) -> AsyncIterator[bytes]:
+        url = self._get_monitoring_url(cluster_name) / id / "log"
+        if since is not None:
+            if since.tzinfo is None:
+                # Interpret naive datetime object as local time.
+                since = since.astimezone(timezone.utc)
+            url = url.update_query(since=since.isoformat())
+        if timestamps:
+            url = url.update_query(timestamps="true")
+        if separator is not None:
+            url = url.update_query(separator=separator)
+        if debug:
+            url = url.update_query(debug="true")
         timeout = attr.evolve(self._core.timeout, sock_read=None)
         auth = await self._config._api_auth()
         async with self._core.request(
@@ -477,14 +540,18 @@ class Jobs(metaclass=NoPublicConstructor):
             ret = await resp.json()
             return ret["tags"]
 
-    async def top(self, id: str) -> AsyncIterator[JobTelemetry]:
-        url = self._config.monitoring_url / id / "top"
+    @asyncgeneratorcontextmanager
+    async def top(
+        self, id: str, *, cluster_name: Optional[str] = None
+    ) -> AsyncIterator[JobTelemetry]:
+        url = self._get_monitoring_url(cluster_name) / id / "top"
         auth = await self._config._api_auth()
         try:
             received_any = False
-            async for resp in self._core.ws_connect(url, auth=auth):
-                yield _job_telemetry_from_api(resp.json())
-                received_any = True
+            async with self._core.ws_connect(url, auth=auth) as ws:
+                async for resp in ws:
+                    yield _job_telemetry_from_api(resp.json())
+                    received_any = True
             if not received_any:
                 raise ValueError(f"Job is not running. Job Id = {id}")
         except WSServerHandshakeError as e:
@@ -498,6 +565,7 @@ class Jobs(metaclass=NoPublicConstructor):
         image: RemoteImage,
         *,
         progress: Optional[AbstractDockerImageProgress] = None,
+        cluster_name: Optional[str] = None,
     ) -> None:
         if not _is_in_neuro_registry(image):
             raise ValueError(f"Image `{image}` must be in the neuro registry")
@@ -505,7 +573,7 @@ class Jobs(metaclass=NoPublicConstructor):
             progress = _DummyProgress()
 
         payload = {"container": {"image": _as_repo_str(image)}}
-        url = self._config.monitoring_url / id / "save"
+        url = self._get_monitoring_url(cluster_name) / id / "save"
 
         auth = await self._config._api_auth()
         timeout = attr.evolve(self._core.timeout, sock_read=None)
@@ -536,10 +604,18 @@ class Jobs(metaclass=NoPublicConstructor):
 
     @asynccontextmanager
     async def port_forward(
-        self, id: str, local_port: int, job_port: int, *, no_key_check: bool = False
+        self,
+        id: str,
+        local_port: int,
+        job_port: int,
+        *,
+        no_key_check: bool = False,
+        cluster_name: Optional[str] = None,
     ) -> AsyncIterator[None]:
         srv = await asyncio.start_server(
-            partial(self._port_forward, id=id, job_port=job_port),
+            partial(
+                self._port_forward, id=id, job_port=job_port, cluster_name=cluster_name
+            ),
             "localhost",
             local_port,
         )
@@ -556,10 +632,13 @@ class Jobs(metaclass=NoPublicConstructor):
         writer: asyncio.StreamWriter,
         id: str,
         job_port: int,
+        *,
+        cluster_name: Optional[str] = None,
     ) -> None:
         try:
             loop = asyncio.get_event_loop()
-            url = self._config.monitoring_url / id / "port_forward" / str(job_port)
+            url = self._get_monitoring_url(cluster_name)
+            url = url / id / "port_forward" / str(job_port)
             auth = await self._config._api_auth()
             ws = await self._core._session.ws_connect(
                 url,
@@ -620,102 +699,92 @@ class Jobs(metaclass=NoPublicConstructor):
         self,
         id: str,
         *,
+        tty: bool = False,
         stdin: bool = False,
         stdout: bool = False,
         stderr: bool = False,
-        logs: bool = False,
+        cluster_name: Optional[str] = None,
     ) -> AsyncIterator[StdStream]:
-        url = self._config.monitoring_url / id / "attach"
+        url = self._get_monitoring_url(cluster_name) / id / "attach"
         url = url.with_query(
+            tty=str(int(tty)),
             stdin=str(int(stdin)),
             stdout=str(int(stdout)),
             stderr=str(int(stderr)),
-            logs=str(int(logs)),
         )
         auth = await self._config._api_auth()
-        ws = await self._core._session.ws_connect(
-            url,
-            headers={"Authorization": auth},
-            timeout=None,  # type: ignore
-            receive_timeout=None,
-            heartbeat=30,
-        )
+
+        try:
+            ws = await self._core._session.ws_connect(
+                url,
+                headers={
+                    "Authorization": auth,
+                    aiohttp.hdrs.SEC_WEBSOCKET_PROTOCOL: "v2.channels.neu.ro",
+                },
+                timeout=None,  # type: ignore
+                receive_timeout=None,
+                heartbeat=30,
+            )
+        except aiohttp.ClientResponseError as ex:
+            if ex.status == 404:
+                raise ResourceNotFound(f"Job {id!r} is not running")
+            raise
 
         try:
             yield StdStream(ws)
         finally:
             await ws.close()
 
-    async def resize(self, id: str, *, w: int, h: int) -> None:
-        url = self._config.monitoring_url / id / "resize"
-        url = url.with_query(w=w, h=h)
-        auth = await self._config._api_auth()
-        async with self._core.request("POST", url, auth=auth):
-            pass
-
-    async def exec_create(self, id: str, cmd: str, *, tty: bool = False) -> str:
-        payload = {
-            "command": cmd,
-            "stdin": True,
-            "stdout": True,
-            "stderr": True,
-            "tty": tty,
-        }
-        url = self._config.monitoring_url / id / "exec_create"
-        auth = await self._config._api_auth()
-        async with self._core.request("POST", url, json=payload, auth=auth) as resp:
-            ret = await resp.json()
-            return ret["exec_id"]
-
-    async def exec_resize(self, id: str, exec_id: str, *, w: int, h: int) -> None:
-        url = self._config.monitoring_url / id / exec_id / "exec_resize"
-        url = url.with_query(w=w, h=h)
-        auth = await self._config._api_auth()
-        async with self._core.request("POST", url, auth=auth) as resp:
-            resp
-
-    async def exec_inspect(self, id: str, exec_id: str) -> ExecInspect:
-        url = self._config.monitoring_url / id / exec_id / "exec_inspect"
-        auth = await self._config._api_auth()
-        async with self._core.request("GET", url, auth=auth) as resp:
-            data = await resp.json()
-            return ExecInspect(
-                id=data["id"],
-                running=data["running"],
-                exit_code=data["exit_code"],
-                job_id=data["job_id"],
-                tty=data["tty"],
-                entrypoint=data["entrypoint"],
-                command=data["command"],
-            )
-
     @asynccontextmanager
-    async def exec_start(self, id: str, exec_id: str) -> AsyncIterator[StdStream]:
-        url = self._config.monitoring_url / id / exec_id / "exec_start"
+    async def exec(
+        self,
+        id: str,
+        cmd: str,
+        *,
+        tty: bool = False,
+        stdin: bool = False,
+        stdout: bool = False,
+        stderr: bool = False,
+        cluster_name: Optional[str] = None,
+    ) -> AsyncIterator[StdStream]:
+        url = self._get_monitoring_url(cluster_name) / id / "exec"
+        url = url.with_query(
+            cmd=cmd,
+            tty=str(int(tty)),
+            stdin=str(int(stdin)),
+            stdout=str(int(stdout)),
+            stderr=str(int(stderr)),
+        )
         auth = await self._config._api_auth()
 
-        ws = await self._core._session.ws_connect(
-            url,
-            headers={"Authorization": auth},
-            timeout=None,  # type: ignore
-            receive_timeout=None,
-            heartbeat=30,
-        )
+        try:
+            ws = await self._core._session.ws_connect(
+                url,
+                headers={"Authorization": auth},
+                timeout=None,  # type: ignore
+                receive_timeout=None,
+                heartbeat=30,
+            )
+        except aiohttp.ClientResponseError as ex:
+            if ex.status == 404:
+                raise ResourceNotFound(f"Job {id!r} is not running")
+            raise
 
         try:
             yield StdStream(ws)
         finally:
             await ws.close()
 
-    async def send_signal(self, id: str, signal: Union[str, int]) -> None:
-        url = self._config.monitoring_url / id / "kill"
-        url = url.with_query(signal=signal)
+    async def send_signal(self, id: str, *, cluster_name: Optional[str] = None) -> None:
+        url = self._get_monitoring_url(cluster_name) / id / "kill"
         auth = await self._config._api_auth()
         async with self._core.request("POST", url, auth=auth) as resp:
             resp
 
-    async def get_capacity(self) -> Mapping[str, int]:
-        url = self._config.monitoring_url / "capacity"
+    async def get_capacity(
+        self, *, cluster_name: Optional[str] = None
+    ) -> Mapping[str, int]:
+        url = self._get_monitoring_url(cluster_name) / "capacity"
         auth = await self._config._api_auth()
         async with self._core.request("GET", url, auth=auth) as resp:
             return await resp.json()
@@ -945,11 +1014,16 @@ def _job_description_from_api(res: Dict[str, Any], parse: Parser) -> JobDescript
         life_span=life_span,
         schedule_timeout=res.get("schedule_timeout", None),
         preset_name=res.get("preset_name"),
+        _internal=JobDescriptionInternal(
+            materialized=res.get("materialized", False),
+            being_dropped=res.get("being_dropped", False),
+            logs_removed=res.get("logs_removed", False),
+        ),
     )
 
 
 def _job_to_api(
-    config: Config,
+    cluster_name: str,
     name: Optional[str] = None,
     preset_name: Optional[str] = None,
     tags: Sequence[str] = (),
@@ -980,7 +1054,7 @@ def _job_to_api(
         primitive["wait_for_jobs_quota"] = wait_for_jobs_quota
     if privileged:
         primitive["privileged"] = privileged
-    primitive["cluster_name"] = config.cluster_name
+    primitive["cluster_name"] = cluster_name
     return primitive
 
 
