@@ -1,3 +1,4 @@
+from datetime import datetime, timezone
 from decimal import Decimal
 from typing import Callable
 
@@ -7,15 +8,17 @@ from aiohttp.web_exceptions import HTTPOk
 
 from neuro_sdk import Client
 from neuro_sdk.admin import (
+    _Balance,
     _CloudProvider,
     _Cluster,
     _ClusterUser,
     _ClusterUserRoleType,
     _NodePool,
+    _Quota,
     _Storage,
+    _UserInfo,
 )
 from neuro_sdk.server_cfg import Preset
-from neuro_sdk.users import Quota
 
 from tests import _TestServerFactory
 
@@ -284,13 +287,50 @@ async def test_list_cluster_users_explicit_cluster(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
     requested_clusters = []
+    date = datetime.now(timezone.utc)
 
     async def handle_list_cluster_user(request: web.Request) -> web.StreamResponse:
         requested_clusters.append(request.match_info["cluster_name"])
         data = [
-            {"user_name": "denis", "role": "admin"},
-            {"user_name": "andrew", "role": "manager"},
-            {"user_name": "ivan", "role": "user"},
+            {
+                "user_name": "denis",
+                "role": "admin",
+                "user_info": {
+                    "first_name": "denis",
+                    "last_name": "admin",
+                    "email": "denis@domain.name",
+                    "created_at": date.isoformat(),
+                },
+                "balance": {
+                    "spent_credits": "0",
+                },
+            },
+            {
+                "user_name": "andrew",
+                "role": "manager",
+                "user_info": {
+                    "first_name": "andrew",
+                    "last_name": "manager",
+                    "email": "andrew@domain.name",
+                    "created_at": date.isoformat(),
+                },
+                "balance": {
+                    "spent_credits": "0",
+                },
+            },
+            {
+                "user_name": "ivan",
+                "role": "user",
+                "user_info": {
+                    "first_name": "ivan",
+                    "last_name": "user",
+                    "email": "ivan@domain.name",
+                    "created_at": date.isoformat(),
+                },
+                "balance": {
+                    "spent_credits": "0",
+                },
+            },
         ]
         return web.json_response(data)
 
@@ -305,13 +345,40 @@ async def test_list_cluster_users_explicit_cluster(
         resp = await client._admin.list_cluster_users("my_cluster")
         assert resp == [
             _ClusterUser(
-                user_name="denis", role=_ClusterUserRoleType("admin"), quota=Quota()
+                user_name="denis",
+                role=_ClusterUserRoleType("admin"),
+                quota=_Quota(),
+                balance=_Balance(),
+                user_info=_UserInfo(
+                    first_name="denis",
+                    last_name="admin",
+                    email="denis@domain.name",
+                    created_at=date,
+                ),
             ),
             _ClusterUser(
-                user_name="andrew", role=_ClusterUserRoleType("manager"), quota=Quota()
+                user_name="andrew",
+                role=_ClusterUserRoleType("manager"),
+                quota=_Quota(),
+                balance=_Balance(),
+                user_info=_UserInfo(
+                    first_name="andrew",
+                    last_name="manager",
+                    email="andrew@domain.name",
+                    created_at=date,
+                ),
             ),
             _ClusterUser(
-                user_name="ivan", role=_ClusterUserRoleType("user"), quota=Quota()
+                user_name="ivan",
+                role=_ClusterUserRoleType("user"),
+                quota=_Quota(),
+                balance=_Balance(),
+                user_info=_UserInfo(
+                    first_name="ivan",
+                    last_name="user",
+                    email="ivan@domain.name",
+                    created_at=date,
+                ),
             ),
         ]
         assert requested_clusters == ["my_cluster"]
@@ -321,13 +388,50 @@ async def test_list_cluster_users_default_cluster(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
     requested_clusters = []
+    date = datetime.now(timezone.utc)
 
     async def handle_list_cluster_user(request: web.Request) -> web.StreamResponse:
         requested_clusters.append(request.match_info["cluster_name"])
         data = [
-            {"user_name": "denis", "role": "admin"},
-            {"user_name": "andrew", "role": "manager"},
-            {"user_name": "ivan", "role": "user"},
+            {
+                "user_name": "denis",
+                "role": "admin",
+                "user_info": {
+                    "first_name": "denis",
+                    "last_name": "admin",
+                    "email": "denis@domain.name",
+                    "created_at": date.isoformat(),
+                },
+                "balance": {
+                    "spent_credits": "0",
+                },
+            },
+            {
+                "user_name": "andrew",
+                "role": "manager",
+                "user_info": {
+                    "first_name": "andrew",
+                    "last_name": "manager",
+                    "email": "andrew@domain.name",
+                    "created_at": date.isoformat(),
+                },
+                "balance": {
+                    "spent_credits": "0",
+                },
+            },
+            {
+                "user_name": "ivan",
+                "role": "user",
+                "user_info": {
+                    "first_name": "ivan",
+                    "last_name": "user",
+                    "email": "ivan@domain.name",
+                    "created_at": date.isoformat(),
+                },
+                "balance": {
+                    "spent_credits": "0",
+                },
+            },
         ]
         return web.json_response(data)
 
@@ -342,13 +446,40 @@ async def test_list_cluster_users_default_cluster(
         resp = await client._admin.list_cluster_users()
         assert resp == [
             _ClusterUser(
-                user_name="denis", role=_ClusterUserRoleType("admin"), quota=Quota()
+                user_name="denis",
+                role=_ClusterUserRoleType("admin"),
+                quota=_Quota(),
+                balance=_Balance(),
+                user_info=_UserInfo(
+                    first_name="denis",
+                    last_name="admin",
+                    email="denis@domain.name",
+                    created_at=date,
+                ),
             ),
             _ClusterUser(
-                user_name="andrew", role=_ClusterUserRoleType("manager"), quota=Quota()
+                user_name="andrew",
+                role=_ClusterUserRoleType("manager"),
+                quota=_Quota(),
+                balance=_Balance(),
+                user_info=_UserInfo(
+                    first_name="andrew",
+                    last_name="manager",
+                    email="andrew@domain.name",
+                    created_at=date,
+                ),
             ),
             _ClusterUser(
-                user_name="ivan", role=_ClusterUserRoleType("user"), quota=Quota()
+                user_name="ivan",
+                role=_ClusterUserRoleType("user"),
+                quota=_Quota(),
+                balance=_Balance(),
+                user_info=_UserInfo(
+                    first_name="ivan",
+                    last_name="user",
+                    email="ivan@domain.name",
+                    created_at=date,
+                ),
             ),
         ]
         assert requested_clusters == ["default"]
@@ -359,11 +490,24 @@ async def test_get_cluster_user(
 ) -> None:
     requested_clusters = []
     requested_users = []
+    date = datetime.now(timezone.utc)
 
     async def handle_get_cluster_user(request: web.Request) -> web.StreamResponse:
         requested_clusters.append(request.match_info["cluster_name"])
         requested_users.append(request.match_info["username"])
-        data = {"user_name": "denis", "role": "admin"}
+        data = {
+            "user_name": "denis",
+            "role": "admin",
+            "user_info": {
+                "first_name": "denis",
+                "last_name": "admin",
+                "email": "denis@domain.name",
+                "created_at": date.isoformat(),
+            },
+            "balance": {
+                "spent_credits": "0",
+            },
+        }
         return web.json_response(data)
 
     app = web.Application()
@@ -377,7 +521,16 @@ async def test_get_cluster_user(
     async with make_client(srv.make_url("/api/v1")) as client:
         resp = await client._admin.get_cluster_user("default", "test")
         assert resp == _ClusterUser(
-            user_name="denis", role=_ClusterUserRoleType("admin"), quota=Quota()
+            user_name="denis",
+            role=_ClusterUserRoleType("admin"),
+            quota=_Quota(),
+            balance=_Balance(),
+            user_info=_UserInfo(
+                first_name="denis",
+                last_name="admin",
+                email="denis@domain.name",
+                created_at=date,
+            ),
         )
         assert requested_clusters == ["default"]
         assert requested_users == ["test"]
@@ -388,11 +541,25 @@ async def test_add_cluster_user(
 ) -> None:
     requested_clusters = []
     requested_payloads = []
+    date = datetime.now(timezone.utc)
 
     async def handle_add_cluster_user(request: web.Request) -> web.StreamResponse:
         payload = await request.json()
         requested_clusters.append(request.match_info["cluster_name"])
         requested_payloads.append(payload)
+        payload = {
+            "user_name": "ivan",
+            "role": "user",
+            "user_info": {
+                "first_name": "ivan",
+                "last_name": "user",
+                "email": "ivan@domain.name",
+                "created_at": date.isoformat(),
+            },
+            "balance": {
+                "spent_credits": "0",
+            },
+        }
         return web.json_response(payload, status=HTTPCreated.status_code)
 
     app = web.Application()
@@ -405,7 +572,16 @@ async def test_add_cluster_user(
     async with make_client(srv.make_url("/api/v1")) as client:
         resp = await client._admin.add_cluster_user("default", "ivan", "user")
         assert resp == _ClusterUser(
-            user_name="ivan", role=_ClusterUserRoleType("user"), quota=Quota()
+            user_name="ivan",
+            role=_ClusterUserRoleType("user"),
+            quota=_Quota(),
+            balance=_Balance(),
+            user_info=_UserInfo(
+                first_name="ivan",
+                last_name="user",
+                email="ivan@domain.name",
+                created_at=date,
+            ),
         )
         assert requested_clusters == ["default"]
         assert requested_payloads == [{"role": "user", "user_name": "ivan"}]
@@ -457,6 +633,10 @@ async def test_set_user_quota(
         requested_payloads.append(dict(payload))
         payload["role"] = "user"
         payload["user_name"] = request.match_info["user_name"]
+        payload["user_info"] = {"email": f"{payload['user_name']}@domain.name"}
+        payload["balance"] = {
+            "spent_credits": "0",
+        }
         return web.json_response(payload, status=HTTPOk.status_code)
 
     app = web.Application()
@@ -468,72 +648,112 @@ async def test_set_user_quota(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/api/v1")) as client:
-        await client._admin.set_user_quota("default", "ivan", Decimal("1000"), 10)
-        await client._admin.set_user_quota("neuro", "user2", None, None)
-        await client._admin.set_user_quota("neuro", "user3", 0, None)
-        assert requested_cluster_users == [
-            ("default", "ivan"),
-            ("neuro", "user2"),
-            ("neuro", "user3"),
-        ]
-        assert len(requested_payloads) == 3
-        assert {
-            "quota": {
-                "credits": "1000",
-                "total_running_jobs": 10,
-            },
-        } in requested_payloads
-        assert {"quota": {}} in requested_payloads
-        assert {
-            "quota": {
-                "credits": "0",
-            },
-        } in requested_payloads
-
-
-async def test_add_user_quota(
-    aiohttp_server: _TestServerFactory, make_client: _MakeClient
-) -> None:
-    requested_cluster_users = []
-    requested_payloads = []
-
-    async def handle_cluster_user_quota_patch(
-        request: web.Request,
-    ) -> web.StreamResponse:
-        requested_cluster_users.append(
-            (
-                request.match_info["cluster_name"],
-                request.match_info["user_name"],
-            )
-        )
-        payload = await request.json()
-        requested_payloads.append(dict(payload))
-        payload["role"] = "user"
-        payload["user_name"] = request.match_info["user_name"]
-        return web.json_response(payload, status=HTTPOk.status_code)
-
-    app = web.Application()
-    app.router.add_patch(
-        "/apis/admin/v1/clusters/{cluster_name}/users/{user_name}/quota",
-        handle_cluster_user_quota_patch,
-    )
-
-    srv = await aiohttp_server(app)
-
-    async with make_client(srv.make_url("/api/v1")) as client:
-        await client._admin.add_user_quota("default", "ivan", Decimal("1000"))
-        await client._admin.add_user_quota("neuro", "user2", None)
+        await client._admin.set_user_quota("default", "ivan", 10)
+        await client._admin.set_user_quota("neuro", "user2", None)
         assert requested_cluster_users == [
             ("default", "ivan"),
             ("neuro", "user2"),
         ]
         assert len(requested_payloads) == 2
         assert {
-            "additional_quota": {
-                "credits": "1000",
+            "quota": {
+                "total_running_jobs": 10,
             },
         } in requested_payloads
-        assert {"additional_quota": {}} in requested_payloads
+        assert {"quota": {}} in requested_payloads
+
+
+async def test_set_user_credits(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    requested_cluster_users = []
+    requested_payloads = []
+
+    async def handle_cluster_user_balance_patch(
+        request: web.Request,
+    ) -> web.StreamResponse:
+        user_name = request.match_info["user_name"]
+        requested_cluster_users.append(
+            (
+                request.match_info["cluster_name"],
+                user_name,
+            )
+        )
+        payload = await request.json()
+        requested_payloads.append(dict(payload))
+        payload = {
+            "role": "user",
+            "user_name": user_name,
+            "user_info": {"email": f"{user_name}@domain.name"},
+            "balance": {
+                "spent_credits": "0",
+            },
+        }
+        return web.json_response(payload, status=HTTPOk.status_code)
+
+    app = web.Application()
+    app.router.add_patch(
+        "/apis/admin/v1/clusters/{cluster_name}/users/{user_name}/balance",
+        handle_cluster_user_balance_patch,
+    )
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/api/v1")) as client:
+        await client._admin.set_user_credits("default", "ivan", Decimal(10))
+        await client._admin.set_user_credits("neuro", "user2", None)
+        assert requested_cluster_users == [
+            ("default", "ivan"),
+            ("neuro", "user2"),
+        ]
+        assert len(requested_payloads) == 2
+        assert {"credits": "10"} in requested_payloads
+        assert {"credits": None} in requested_payloads
+
+
+async def test_add_user_credits(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    requested_cluster_users = []
+    requested_payloads = []
+
+    async def handle_cluster_user_balance_patch(
+        request: web.Request,
+    ) -> web.StreamResponse:
+        user_name = request.match_info["user_name"]
+        requested_cluster_users.append(
+            (
+                request.match_info["cluster_name"],
+                user_name,
+            )
+        )
+        payload = await request.json()
+        requested_payloads.append(dict(payload))
+        payload = {
+            "role": "user",
+            "user_name": user_name,
+            "user_info": {"email": f"{user_name}@domain.name"},
+            "balance": {
+                "spent_credits": "0",
+            },
+        }
+        return web.json_response(payload, status=HTTPOk.status_code)
+
+    app = web.Application()
+    app.router.add_patch(
+        "/apis/admin/v1/clusters/{cluster_name}/users/{user_name}/balance",
+        handle_cluster_user_balance_patch,
+    )
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/api/v1")) as client:
+        await client._admin.add_user_credits("default", "ivan", Decimal("1000"))
+        assert requested_cluster_users == [
+            ("default", "ivan"),
+        ]
+        assert len(requested_payloads) == 1
+        assert {"additional_credits": "1000"} in requested_payloads
 
 
 async def test_get_cloud_provider_options(
