@@ -1,6 +1,7 @@
 import asyncio
 import json
 import sys
+from datetime import datetime, timezone
 from typing import Any, Callable, Dict, List, Optional
 
 import pytest
@@ -46,6 +47,8 @@ async def test_jobs_monitor(
 ) -> None:
     async def log_stream(request: web.Request) -> web.StreamResponse:
         assert request.headers["Accept-Encoding"] == "identity"
+        assert "since" not in request.query
+        assert request.query.get("timestamps", "false") == "false"
         resp = web.StreamResponse()
         resp.enable_chunked_encoding()
         resp.enable_compression(web.ContentCoding.identity)
@@ -61,8 +64,9 @@ async def test_jobs_monitor(
 
     lst = []
     async with make_client(srv.make_url("/")) as client:
-        async for data in client.jobs.monitor("job-id"):
-            lst.append(data)
+        async with client.jobs.monitor("job-id") as it:
+            async for data in it:
+                lst.append(data)
 
     assert b"".join(lst) == b"".join(
         [
@@ -80,6 +84,87 @@ async def test_jobs_monitor(
     )
 
 
+async def test_jobs_monitor_since(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def log_stream(request: web.Request) -> web.StreamResponse:
+        assert request.headers["Accept-Encoding"] == "identity"
+        assert request.query["since"] == "2021-08-17T00:00:00+00:00"
+        assert request.query.get("timestamps", "false") == "false"
+        resp = web.StreamResponse()
+        resp.enable_chunked_encoding()
+        resp.enable_compression(web.ContentCoding.identity)
+        await resp.prepare(request)
+        for i in range(5, 10):
+            await resp.write(b"chunk " + str(i).encode("ascii") + b"\n")
+        return resp
+
+    app = web.Application()
+    app.router.add_get("/jobs/job-id/log", log_stream)
+
+    srv = await aiohttp_server(app)
+
+    lst = []
+    async with make_client(srv.make_url("/")) as client:
+        async with client.jobs.monitor(
+            "job-id", since=datetime(2021, 8, 17, tzinfo=timezone.utc)
+        ) as it:
+            async for data in it:
+                lst.append(data)
+
+    assert b"".join(lst) == b"".join(
+        [
+            b"chunk 5\n",
+            b"chunk 6\n",
+            b"chunk 7\n",
+            b"chunk 8\n",
+            b"chunk 9\n",
+        ]
+    )
+
+
+async def test_jobs_monitor_timestamps(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def log_stream(request: web.Request) -> web.StreamResponse:
+        assert request.headers["Accept-Encoding"] == "identity"
+        assert "since" not in request.query
+        assert request.query["timestamps"] == "true"
+        resp = web.StreamResponse()
+        resp.enable_chunked_encoding()
+        resp.enable_compression(web.ContentCoding.identity)
+        await resp.prepare(request)
+        for i in range(10):
+            await resp.write(f"2021-08-13T09:23:{i:02}.123456789Z chunk {i}\n".encode())
+        return resp
+
+    app = web.Application()
+    app.router.add_get("/jobs/job-id/log", log_stream)
+
+    srv = await aiohttp_server(app)
+
+    lst = []
+    async with make_client(srv.make_url("/")) as client:
+        async with client.jobs.monitor("job-id", timestamps=True) as it:
+            async for data in it:
+                lst.append(data)
+
+    assert b"".join(lst) == b"".join(
+        [
+            b"2021-08-13T09:23:00.123456789Z chunk 0\n",
+            b"2021-08-13T09:23:01.123456789Z chunk 1\n",
+            b"2021-08-13T09:23:02.123456789Z chunk 2\n",
+            b"2021-08-13T09:23:03.123456789Z chunk 3\n",
+            b"2021-08-13T09:23:04.123456789Z chunk 4\n",
+            b"2021-08-13T09:23:05.123456789Z chunk 5\n",
+            b"2021-08-13T09:23:06.123456789Z chunk 6\n",
+            b"2021-08-13T09:23:07.123456789Z chunk 7\n",
+            b"2021-08-13T09:23:08.123456789Z chunk 8\n",
+            b"2021-08-13T09:23:09.123456789Z chunk 9\n",
+        ]
+    )
+
+
 async def test_monitor_notexistent_job(
     aiohttp_server: Any, make_client: _MakeClient
 ) -> None:
@@ -94,8 +179,9 @@ async def test_monitor_notexistent_job(
     lst = []
     async with make_client(srv.make_url("/")) as client:
         with pytest.raises(ResourceNotFound):
-            async for data in client.jobs.monitor("job-id"):
-                lst.append(data)
+            async with client.jobs.monitor("job-id") as it:
+                async for data in it:
+                    lst.append(data)
     assert lst == []
 
 
@@ -133,8 +219,9 @@ async def test_job_top(
 
     lst = []
     async with make_client(srv.make_url("/")) as client:
-        async for data in client.jobs.top("job-id"):
-            lst.append(data)
+        async with client.jobs.top("job-id") as it:
+            async for data in it:
+                lst.append(data)
 
     assert lst == [get_job_telemetry(i) for i in range(10)]
 
@@ -157,8 +244,9 @@ async def test_top_finished_job(
     lst = []
     async with make_client(srv.make_url("/")) as client:
         with pytest.raises(ValueError, match="not running"):
-            async for data in client.jobs.top("job-id"):
-                lst.append(data)
+            async with client.jobs.top("job-id") as it:
+                async for data in it:
+                    lst.append(data)
     assert lst == []
 
 
@@ -176,8 +264,9 @@ async def test_top_nonexisting_job(
     lst = []
     async with make_client(srv.make_url("/")) as client:
         with pytest.raises(ValueError, match="not found"):
-            async for data in client.jobs.top("job-id"):
-                lst.append(data)
+            async with client.jobs.top("job-id") as it:
+                async for data in it:
+                    lst.append(data)
     assert lst == []
 
 
@@ -513,6 +602,71 @@ async def test_status_failed(
         assert ret.preemptible_node
 
 
+async def test_status_being_dropped(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    JSON = {
+        "status": "failed",
+        "id": "job-id",
+        "description": "This is job description, not a history description",
+        "http_url": "http://my_host:8889",
+        "history": {
+            "created_at": "2018-08-29T12:23:13.981621+00:00",
+            "started_at": "2018-08-29T12:23:15.988054+00:00",
+            "finished_at": "2018-08-29T12:59:31.427795+00:00",
+            "reason": "ContainerCannotRun",
+            "description": "Not enough coffee",
+        },
+        "scheduler_enabled": True,
+        "preemptible_node": True,
+        "pass_config": False,
+        "being_dropped": True,
+        "logs_removed": True,
+        "owner": "owner",
+        "cluster_name": "default",
+        "uri": "job://default/owner/job-id",
+        "container": {
+            "image": "submit-image-name",
+            "command": "submit-command",
+            "http": {"port": 8181},
+            "resources": {
+                "memory_mb": "4096",
+                "cpu": 7.0,
+                "shm": True,
+                "gpu": 1,
+                "gpu_model": "test-gpu-model",
+            },
+            "volumes": [
+                {
+                    "src_storage_uri": "storage://test-user/path_read_only",
+                    "dst_path": "/container/read_only",
+                    "read_only": True,
+                },
+                {
+                    "src_storage_uri": "storage://test-user/path_read_write",
+                    "dst_path": "/container/path_read_write",
+                    "read_only": False,
+                },
+            ],
+        },
+    }
+
+    async def handler(request: web.Request) -> web.Response:
+        return web.json_response(JSON)
+
+    app = web.Application()
+    app.router.add_get("/jobs/job-id", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        ret = await client.jobs.status("job-id")
+
+        assert ret == _job_description_from_api(JSON, client.parse)
+        assert ret._internal.being_dropped
+        assert ret._internal.logs_removed
+
+
 async def test_status_with_http(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -1893,8 +2047,9 @@ async def test_list_error_in_stream_response(
 
     async with make_client(srv.make_url("/")) as client:
         with pytest.raises(Exception) as err:
-            async for _ in client.jobs.list():
-                pass
+            async with client.jobs.list() as it:
+                async for _ in it:
+                    pass
         assert err.value.args[0] == "Failed"
 
 
@@ -1917,7 +2072,8 @@ async def test_list_no_filter(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        ret = [job async for job in client.jobs.list()]
+        async with client.jobs.list() as it:
+            ret = [job async for job in it]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1955,7 +2111,8 @@ async def test_list_filter_by_name(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        ret = [job async for job in client.jobs.list(name=name_1)]
+        async with client.jobs.list(name=name_1) as it:
+            ret = [job async for job in it]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -1995,7 +2152,8 @@ async def test_list_filter_by_statuses(
 
     statuses = {JobStatus.FAILED, JobStatus.SUCCEEDED, JobStatus.CANCELLED}
     async with make_client(srv.make_url("/")) as client:
-        ret = [job async for job in client.jobs.list(statuses=statuses)]
+        async with client.jobs.list(statuses=statuses) as it:
+            ret = [job async for job in it]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -2027,7 +2185,8 @@ async def test_list_incorrect_image(
 
     statuses = {JobStatus.FAILED, JobStatus.SUCCEEDED}
     async with make_client(srv.make_url("/")) as client:
-        ret = [job async for job in client.jobs.list(statuses=statuses)]
+        async with client.jobs.list(statuses=statuses) as it:
+            ret = [job async for job in it]
     for job in ret:
         if job.status == JobStatus.FAILED:
             assert job.container.image.name == INVALID_IMAGE_NAME
@@ -2152,7 +2311,8 @@ async def test_list_filter_by_name_and_statuses(
     statuses = {JobStatus.PENDING, JobStatus.SUCCEEDED}
     name = "job-name-1"
     async with make_client(srv.make_url("/")) as client:
-        ret = [job async for job in client.jobs.list(statuses=statuses, name=name)]
+        async with client.jobs.list(statuses=statuses, name=name) as it:
+            ret = [job async for job in it]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -2189,7 +2349,8 @@ async def test_list_filter_by_tags(
 
     tags = {"t1", "t2"}
     async with make_client(srv.make_url("/")) as client:
-        ret = [job async for job in client.jobs.list(tags=tags)]
+        async with client.jobs.list(tags=tags) as it:
+            ret = [job async for job in it]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -2236,12 +2397,8 @@ async def test_list_filter_by_name_and_statuses_and_owners(
     name = name_1
     owners = {owner_1, owner_2}
     async with make_client(srv.make_url("/")) as client:
-        ret = [
-            job
-            async for job in client.jobs.list(
-                statuses=statuses, name=name, owners=owners
-            )
-        ]
+        async with client.jobs.list(statuses=statuses, name=name, owners=owners) as it:
+            ret = [job async for job in it]
 
         job_descriptions = [
             _job_description_from_api(job, client.parse) for job in jobs
@@ -2281,25 +2438,32 @@ async def test_list_filter_by_date_range(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        ret = [job async for job in client.jobs.list()]
+        async with client.jobs.list() as it:
+            ret = [job async for job in it]
         assert {job.id for job in ret} == {"job-id-1", "job-id-2", "job-id-3"}
 
-        ret = [job async for job in client.jobs.list(since=t2)]
+        async with client.jobs.list(since=t2) as it:
+            ret = [job async for job in it]
         assert {job.id for job in ret} == {"job-id-2", "job-id-3"}
 
-        ret = [job async for job in client.jobs.list(until=t2)]
+        async with client.jobs.list(until=t2) as it:
+            ret = [job async for job in it]
         assert {job.id for job in ret} == {"job-id-1", "job-id-2"}
 
-        ret = [job async for job in client.jobs.list(since=t1, until=t2)]
+        async with client.jobs.list(since=t1, until=t2) as it:
+            ret = [job async for job in it]
         assert {job.id for job in ret} == {"job-id-1", "job-id-2"}
 
-        ret = [job async for job in client.jobs.list(since=t1, until=t3)]
+        async with client.jobs.list(since=t1, until=t3) as it:
+            ret = [job async for job in it]
         assert {job.id for job in ret} == {"job-id-1", "job-id-2", "job-id-3"}
 
-        ret = [job async for job in client.jobs.list(since=t2naive)]
+        async with client.jobs.list(since=t2naive) as it:
+            ret = [job async for job in it]
         assert {job.id for job in ret} == {"job-id-2", "job-id-3"}
 
-        ret = [job async for job in client.jobs.list(until=t2naive)]
+        async with client.jobs.list(until=t2naive) as it:
+            ret = [job async for job in it]
         assert {job.id for job in ret} == {"job-id-1", "job-id-2"}
 
 
