@@ -24,7 +24,7 @@ from neuro_sdk import (
     StorageProgressStep,
 )
 from neuro_sdk.abc import StorageProgressDelete
-from neuro_sdk.storage import _parse_content_range
+from neuro_sdk.storage import DiskUsageInfo, _parse_content_range
 
 from tests import _RawTestServerFactory, _TestServerFactory
 
@@ -197,10 +197,7 @@ async def test_storage_ls_legacy(
 
     srv = await aiohttp_server(app)
 
-    async with make_client(srv.make_url("/")) as client:
-        ret = [file async for file in client.storage.ls(URL("storage:folder"))]
-
-    assert ret == [
+    expected = [
         FileStatus(
             path="foo",
             size=1024,
@@ -219,6 +216,16 @@ async def test_storage_ls_legacy(
         ),
     ]
 
+    async with make_client(srv.make_url("/")) as client:
+        async with client.storage.list(URL("storage:folder")) as it:
+            ret = [file async for file in it]
+            assert ret == expected
+
+        with pytest.warns(DeprecationWarning):
+            async with client.storage.ls(URL("storage:folder")) as it:
+                ret = [file async for file in it]
+                assert ret == expected
+
 
 async def make_listiter_response(
     request: web.Request, file_statuses: List[Any]
@@ -265,7 +272,8 @@ async def test_storage_ls(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        ret = [file async for file in client.storage.ls(URL("storage:folder"))]
+        async with client.storage.list(URL("storage:folder")) as it:
+            ret = [file async for file in it]
 
     assert ret == [
         FileStatus(
@@ -287,6 +295,46 @@ async def test_storage_ls(
     ]
 
 
+async def test_storage_disk_usage(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def handler(request: web.Request) -> web.StreamResponse:
+        assert "b3" in request.headers
+        assert request.path == "/storage/user"
+        assert request.query == {"op": "GETDISKUSAGE"}
+        return web.json_response({"total": 100, "used": 20, "free": 80})
+
+    app = web.Application()
+    app.router.add_get("/storage/user", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        res = await client.storage.disk_usage()
+
+    assert res == DiskUsageInfo(total=100, used=20, free=80, cluster_name="default")
+
+
+async def test_storage_disk_usage_another_cluster(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def handler(request: web.Request) -> web.StreamResponse:
+        assert "b3" in request.headers
+        assert request.path == "/storage2/user"
+        assert request.query == {"op": "GETDISKUSAGE"}
+        return web.json_response({"total": 100, "used": 20, "free": 80})
+
+    app = web.Application()
+    app.router.add_get("/storage2/user", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        res = await client.storage.disk_usage(cluster_name="another")
+
+    assert res == DiskUsageInfo(total=100, used=20, free=80, cluster_name="another")
+
+
 async def test_storage_ls_another_cluster(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -319,10 +367,8 @@ async def test_storage_ls_another_cluster(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        ret = [
-            file
-            async for file in client.storage.ls(URL("storage://another/user/folder"))
-        ]
+        async with client.storage.list(URL("storage://another/user/folder")) as it:
+            ret = [file async for file in it]
 
     assert ret == [
         FileStatus(
@@ -366,8 +412,9 @@ async def test_storage_ls_error_in_server_response(
 
     async with make_client(srv.make_url("/")) as client:
         with pytest.raises(OSError) as err:
-            async for _ in client.storage.ls(URL("storage:folder")):
-                pass
+            async with client.storage.list(URL("storage:folder")) as it:
+                async for _ in it:
+                    pass
         assert err.value.strerror == "Server is to busy"
         assert err.value.errno == errno.EBUSY
 
@@ -492,7 +539,8 @@ async def test_storage_glob(
     async with make_client(srv.make_url("/")) as client:
 
         async def glob(pattern: str) -> List[URL]:
-            return [uri async for uri in client.storage.glob(URL(pattern))]
+            async with client.storage.glob(URL(pattern)) as it:
+                return [uri async for uri in it]
 
         assert await glob("storage:folder") == [URL("storage:folder")]
         assert await glob("storage:folder/") == [URL("storage:folder/")]
@@ -1112,8 +1160,9 @@ async def test_storage_open(
 
     async with make_client(srv.make_url("/")) as client:
         buf = bytearray()
-        async for chunk in client.storage.open(URL("storage:file")):
-            buf.extend(chunk)
+        async with client.storage.open(URL("storage:file")) as it:
+            async for chunk in it:
+                buf.extend(chunk)
         assert buf == b"01234"
 
 
@@ -1138,8 +1187,9 @@ async def test_storage_open_another_cluster(
 
     async with make_client(srv.make_url("/")) as client:
         buf = bytearray()
-        async for chunk in client.storage.open(URL("storage://another/user/file")):
-            buf.extend(chunk)
+        async with client.storage.open(URL("storage://another/user/file")) as it:
+            async for chunk in it:
+                buf.extend(chunk)
         assert buf == b"01234"
 
 
@@ -1167,23 +1217,27 @@ async def test_storage_open_partial_read(
 
     async with make_client(srv.make_url("/")) as client:
         buf = bytearray()
-        async for chunk in client.storage.open(URL("storage:file"), 5):
-            buf.extend(chunk)
+        async with client.storage.open(URL("storage:file"), 5) as it:
+            async for chunk in it:
+                buf.extend(chunk)
         assert buf == b"halamaha"
 
         buf = bytearray()
-        async for chunk in client.storage.open(URL("storage:file"), 5, 4):
-            buf.extend(chunk)
+        async with client.storage.open(URL("storage:file"), 5, 4) as it:
+            async for chunk in it:
+                buf.extend(chunk)
         assert buf == b"hala"
 
         buf = bytearray()
-        async for chunk in client.storage.open(URL("storage:file"), 5, 20):
-            buf.extend(chunk)
+        async with client.storage.open(URL("storage:file"), 5, 20) as it:
+            async for chunk in it:
+                buf.extend(chunk)
         assert buf == b"halamaha"
 
         buf = bytearray()
-        async for chunk in client.storage.open(URL("storage:file"), 5, 0):
-            buf.extend(chunk)
+        async with client.storage.open(URL("storage:file"), 5, 0) as it:
+            async for chunk in it:
+                buf.extend(chunk)
         assert buf == b""
 
 
@@ -1208,13 +1262,15 @@ async def test_storage_open_unsupported_partial_read(
 
     async with make_client(srv.make_url("/")) as client:
         buf = bytearray()
-        async for chunk in client.storage.open(URL("storage:file"), 0):
-            buf.extend(chunk)
+        async with client.storage.open(URL("storage:file"), 0) as it:
+            async for chunk in it:
+                buf.extend(chunk)
         assert buf == b"01234"
 
         with pytest.raises(RuntimeError):
-            async for chunk in client.storage.open(URL("storage:file"), 5):
-                pass
+            async with client.storage.open(URL("storage:file"), 5) as it:
+                async for chunk in it:
+                    pass
 
 
 async def test_storage_open_directory(
@@ -1243,8 +1299,9 @@ async def test_storage_open_directory(
     async with make_client(srv.make_url("/")) as client:
         buf = bytearray()
         with pytest.raises((IsADirectoryError, IllegalArgumentError)):
-            async for chunk in client.storage.open(URL("storage:folder")):
-                buf.extend(chunk)
+            async with client.storage.open(URL("storage:folder")) as it:
+                async for chunk in it:
+                    buf.extend(chunk)
         assert not buf
 
 
@@ -1985,3 +2042,29 @@ async def test_storage_upload_dir_with_ignore_file_names(
     assert names == [".neuroignore", "nested", "three", "two"]
     names = sorted(os.listdir(storage_path / "folder" / "nested"))
     assert names == [".gitignore", "three"]
+
+
+async def test_storage_upload_dir_with_parent_ignore_file_names(
+    storage_server: Any, make_client: _MakeClient, tmp_path: Path, storage_path: Path
+) -> None:
+    parent_dir = tmp_path / "parent"
+    local_dir = parent_dir / "folder"
+    local_dir2 = local_dir / "nested"
+    local_dir2.mkdir(parents=True)
+    for name in "one", "two", "three":
+        (local_dir / name).write_bytes(b"")
+        (local_dir2 / name).write_bytes(b"")
+    (tmp_path / ".neuroignore").write_text("one")
+    (parent_dir / ".gitignore").write_text("*/two")
+
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(
+            URL(local_dir.as_uri()),
+            URL("storage:folder"),
+            ignore_file_names={".neuroignore", ".gitignore"},
+        )
+
+    names = sorted(os.listdir(storage_path / "folder"))
+    assert names == ["nested", "three"]
+    names = sorted(os.listdir(storage_path / "folder" / "nested"))
+    assert names == ["three", "two"]
